@@ -1,58 +1,164 @@
 import inspect
 import io
+from collections.abc import Callable, Sequence
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
-    Dict,
-    List,
     Optional,
-    Sequence,
-    Type,
     TypeVar,
-    Union,
 )
 
 import click
-
-from ._compat_utils import _get_click_major
+import click.shell_completion
 
 if TYPE_CHECKING:  # pragma: no cover
-    if _get_click_major() > 7:
-        import click.shell_completion
-
     from .core import TyperCommand, TyperGroup
     from .main import Typer
 
 
 NoneType = type(None)
 
-AnyType = Type[Any]
+AnyType = type[Any]
 
 Required = ...
 
 
 class Context(click.Context):
+    """
+    The [`Context`](https://click.palletsprojects.com/en/stable/api/#click.Context) has some additional data about the current execution of your program.
+    When declaring it in a [callback](https://typer.tiangolo.com/tutorial/options/callback-and-context/) function,
+    you can access this additional information.
+    """
+
     pass
 
 
 class FileText(io.TextIOWrapper):
+    """
+    Gives you a file-like object for reading text, and you will get a `str` data from it.
+    The default mode of this class is `mode="r"`.
+
+    **Example**
+
+    ```python
+    from typing import Annotated
+
+    import typer
+
+    app = typer.Typer()
+
+    @app.command()
+    def main(config: Annotated[typer.FileText, typer.Option()]):
+        for line in config:
+            print(f"Config line: {line}")
+
+    if __name__ == "__main__":
+        app()
+    ```
+    """
+
     pass
 
 
 class FileTextWrite(FileText):
+    """
+    You can use this class for writing text. Alternatively, you can use `FileText` with `mode="w"`.
+    The default mode of this class is `mode="w"`.
+
+    **Example**
+
+    ```python
+    from typing import Annotated
+
+    import typer
+
+    app = typer.Typer()
+
+    @app.command()
+    def main(config: Annotated[typer.FileTextWrite, typer.Option()]):
+        config.write("Some config written by the app")
+        print("Config written")
+
+    if __name__ == "__main__":
+        app()
+    ```
+    """
+
     pass
 
 
 class FileBinaryRead(io.BufferedReader):
+    """
+    You can use this class to read binary data, receiving `bytes`.
+    The default mode of this class is `mode="rb"`.
+    It is useful for reading binary files like images:
+
+    **Example**
+
+    ```python
+    from typing import Annotated
+
+    import typer
+
+    app = typer.Typer()
+
+    @app.command()
+    def main(file: Annotated[typer.FileBinaryRead, typer.Option()]):
+        processed_total = 0
+        for bytes_chunk in file:
+            # Process the bytes in bytes_chunk
+            processed_total += len(bytes_chunk)
+            print(f"Processed bytes total: {processed_total}")
+
+    if __name__ == "__main__":
+        app()
+    ```
+    """
+
     pass
 
 
 class FileBinaryWrite(io.BufferedWriter):
+    """
+    You can use this class to write binary data: you pass `bytes` to it instead of strings.
+    The default mode of this class is `mode="wb"`.
+    It is useful for writing binary files like images:
+
+    **Example**
+
+    ```python
+    from typing import Annotated
+
+    import typer
+
+    app = typer.Typer()
+
+    @app.command()
+    def main(file: Annotated[typer.FileBinaryWrite, typer.Option()]):
+        first_line_str = "some settings\\n"
+        # You cannot write str directly to a binary file; encode it first
+        first_line_bytes = first_line_str.encode("utf-8")
+        # Then you can write the bytes
+        file.write(first_line_bytes)
+        # This is already bytes, it starts with b"
+        second_line = b"la cig\xc3\xbce\xc3\xb1a trae al ni\xc3\xb1o"
+        file.write(second_line)
+        print("Binary file written")
+
+    if __name__ == "__main__":
+        app()
+    ```
+    """
+
     pass
 
 
 class CallbackParam(click.Parameter):
+    """
+    In a callback function, you can declare a function parameter with type `CallbackParam`
+    to access the specific Click [`Parameter`](https://click.palletsprojects.com/en/stable/api/#click.Parameter) object.
+    """
+
     pass
 
 
@@ -89,21 +195,21 @@ def Default(value: DefaultType) -> DefaultType:
 class CommandInfo:
     def __init__(
         self,
-        name: Optional[str] = None,
+        name: str | None = None,
         *,
-        cls: Optional[Type["TyperCommand"]] = None,
-        context_settings: Optional[Dict[Any, Any]] = None,
-        callback: Optional[Callable[..., Any]] = None,
-        help: Optional[str] = None,
-        epilog: Optional[str] = None,
-        short_help: Optional[str] = None,
+        cls: type["TyperCommand"] | None = None,
+        context_settings: dict[Any, Any] | None = None,
+        callback: Callable[..., Any] | None = None,
+        help: str | None = None,
+        epilog: str | None = None,
+        short_help: str | None = None,
         options_metavar: str = "[OPTIONS]",
         add_help_option: bool = True,
         no_args_is_help: bool = False,
         hidden: bool = False,
         deprecated: bool = False,
         # Rich settings
-        rich_help_panel: Union[str, None] = None,
+        rich_help_panel: str | None = None,
     ):
         self.name = name
         self.cls = cls
@@ -126,25 +232,25 @@ class TyperInfo:
         self,
         typer_instance: Optional["Typer"] = Default(None),
         *,
-        name: Optional[str] = Default(None),
-        cls: Optional[Type["TyperGroup"]] = Default(None),
+        name: str | None = Default(None),
+        cls: type["TyperGroup"] | None = Default(None),
         invoke_without_command: bool = Default(False),
         no_args_is_help: bool = Default(False),
-        subcommand_metavar: Optional[str] = Default(None),
+        subcommand_metavar: str | None = Default(None),
         chain: bool = Default(False),
-        result_callback: Optional[Callable[..., Any]] = Default(None),
+        result_callback: Callable[..., Any] | None = Default(None),
         # Command
-        context_settings: Optional[Dict[Any, Any]] = Default(None),
-        callback: Optional[Callable[..., Any]] = Default(None),
-        help: Optional[str] = Default(None),
-        epilog: Optional[str] = Default(None),
-        short_help: Optional[str] = Default(None),
+        context_settings: dict[Any, Any] | None = Default(None),
+        callback: Callable[..., Any] | None = Default(None),
+        help: str | None = Default(None),
+        epilog: str | None = Default(None),
+        short_help: str | None = Default(None),
         options_metavar: str = Default("[OPTIONS]"),
         add_help_option: bool = Default(True),
         hidden: bool = Default(False),
         deprecated: bool = Default(False),
         # Rich settings
-        rich_help_panel: Union[str, None] = Default(None),
+        rich_help_panel: str | None = Default(None),
     ):
         self.typer_instance = typer_instance
         self.name = name
@@ -170,43 +276,44 @@ class ParameterInfo:
     def __init__(
         self,
         *,
-        default: Optional[Any] = None,
-        param_decls: Optional[Sequence[str]] = None,
-        callback: Optional[Callable[..., Any]] = None,
-        metavar: Optional[str] = None,
+        default: Any | None = None,
+        param_decls: Sequence[str] | None = None,
+        callback: Callable[..., Any] | None = None,
+        metavar: str | None = None,
         expose_value: bool = True,
         is_eager: bool = False,
-        envvar: Optional[Union[str, List[str]]] = None,
-        shell_complete: Optional[
-            Callable[
-                [click.Context, click.Parameter, str],
-                Union[List["click.shell_completion.CompletionItem"], List[str]],
-            ]
-        ] = None,
-        autocompletion: Optional[Callable[..., Any]] = None,
-        default_factory: Optional[Callable[[], Any]] = None,
+        envvar: str | list[str] | None = None,
+        # Note that shell_complete is not fully supported and will be removed in future versions
+        # TODO: Remove shell_complete in a future version (after 0.16.0)
+        shell_complete: Callable[
+            [click.Context, click.Parameter, str],
+            list["click.shell_completion.CompletionItem"] | list[str],
+        ]
+        | None = None,
+        autocompletion: Callable[..., Any] | None = None,
+        default_factory: Callable[[], Any] | None = None,
         # Custom type
-        parser: Optional[Callable[[str], Any]] = None,
-        click_type: Optional[click.ParamType] = None,
+        parser: Callable[[str], Any] | None = None,
+        click_type: click.ParamType | None = None,
         # TyperArgument
-        show_default: Union[bool, str] = True,
+        show_default: bool | str = True,
         show_choices: bool = True,
         show_envvar: bool = True,
-        help: Optional[str] = None,
+        help: str | None = None,
         hidden: bool = False,
         # Choice
         case_sensitive: bool = True,
         # Numbers
-        min: Optional[Union[int, float]] = None,
-        max: Optional[Union[int, float]] = None,
+        min: int | float | None = None,
+        max: int | float | None = None,
         clamp: bool = False,
         # DateTime
-        formats: Optional[List[str]] = None,
+        formats: list[str] | None = None,
         # File
-        mode: Optional[str] = None,
-        encoding: Optional[str] = None,
-        errors: Optional[str] = "strict",
-        lazy: Optional[bool] = None,
+        mode: str | None = None,
+        encoding: str | None = None,
+        errors: str | None = "strict",
+        lazy: bool | None = None,
         atomic: bool = False,
         # Path
         exists: bool = False,
@@ -216,9 +323,9 @@ class ParameterInfo:
         readable: bool = True,
         resolve_path: bool = False,
         allow_dash: bool = False,
-        path_type: Union[None, Type[str], Type[bytes]] = None,
+        path_type: None | type[str] | type[bytes] = None,
         # Rich settings
-        rich_help_panel: Union[str, None] = None,
+        rich_help_panel: str | None = None,
     ):
         # Check if user has provided multiple custom parsers
         if parser and click_type:
@@ -278,51 +385,53 @@ class OptionInfo(ParameterInfo):
         self,
         *,
         # ParameterInfo
-        default: Optional[Any] = None,
-        param_decls: Optional[Sequence[str]] = None,
-        callback: Optional[Callable[..., Any]] = None,
-        metavar: Optional[str] = None,
+        default: Any | None = None,
+        param_decls: Sequence[str] | None = None,
+        callback: Callable[..., Any] | None = None,
+        metavar: str | None = None,
         expose_value: bool = True,
         is_eager: bool = False,
-        envvar: Optional[Union[str, List[str]]] = None,
-        shell_complete: Optional[
-            Callable[
-                [click.Context, click.Parameter, str],
-                Union[List["click.shell_completion.CompletionItem"], List[str]],
-            ]
-        ] = None,
-        autocompletion: Optional[Callable[..., Any]] = None,
-        default_factory: Optional[Callable[[], Any]] = None,
+        envvar: str | list[str] | None = None,
+        # Note that shell_complete is not fully supported and will be removed in future versions
+        # TODO: Remove shell_complete in a future version (after 0.16.0)
+        shell_complete: Callable[
+            [click.Context, click.Parameter, str],
+            list["click.shell_completion.CompletionItem"] | list[str],
+        ]
+        | None = None,
+        autocompletion: Callable[..., Any] | None = None,
+        default_factory: Callable[[], Any] | None = None,
         # Custom type
-        parser: Optional[Callable[[str], Any]] = None,
-        click_type: Optional[click.ParamType] = None,
+        parser: Callable[[str], Any] | None = None,
+        click_type: click.ParamType | None = None,
         # Option
-        show_default: bool = True,
-        prompt: Union[bool, str] = False,
+        show_default: bool | str = True,
+        prompt: bool | str = False,
         confirmation_prompt: bool = False,
         prompt_required: bool = True,
         hide_input: bool = False,
-        is_flag: Optional[bool] = None,
-        flag_value: Optional[Any] = None,
+        # TODO: remove is_flag and flag_value in a future release
+        is_flag: bool | None = None,
+        flag_value: Any | None = None,
         count: bool = False,
         allow_from_autoenv: bool = True,
-        help: Optional[str] = None,
+        help: str | None = None,
         hidden: bool = False,
         show_choices: bool = True,
         show_envvar: bool = True,
         # Choice
         case_sensitive: bool = True,
         # Numbers
-        min: Optional[Union[int, float]] = None,
-        max: Optional[Union[int, float]] = None,
+        min: int | float | None = None,
+        max: int | float | None = None,
         clamp: bool = False,
         # DateTime
-        formats: Optional[List[str]] = None,
+        formats: list[str] | None = None,
         # File
-        mode: Optional[str] = None,
-        encoding: Optional[str] = None,
-        errors: Optional[str] = "strict",
-        lazy: Optional[bool] = None,
+        mode: str | None = None,
+        encoding: str | None = None,
+        errors: str | None = "strict",
+        lazy: bool | None = None,
         atomic: bool = False,
         # Path
         exists: bool = False,
@@ -332,9 +441,9 @@ class OptionInfo(ParameterInfo):
         readable: bool = True,
         resolve_path: bool = False,
         allow_dash: bool = False,
-        path_type: Union[None, Type[str], Type[bytes]] = None,
+        path_type: None | type[str] | type[bytes] = None,
         # Rich settings
-        rich_help_panel: Union[str, None] = None,
+        rich_help_panel: str | None = None,
     ):
         super().__init__(
             default=default,
@@ -382,12 +491,19 @@ class OptionInfo(ParameterInfo):
             # Rich settings
             rich_help_panel=rich_help_panel,
         )
+        if is_flag is not None or flag_value is not None:
+            import warnings
+
+            warnings.warn(
+                "The 'is_flag' and 'flag_value' parameters are not supported by Typer "
+                "and will be removed entirely in a future release.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
         self.prompt = prompt
         self.confirmation_prompt = confirmation_prompt
         self.prompt_required = prompt_required
         self.hide_input = hide_input
-        self.is_flag = is_flag
-        self.flag_value = flag_value
         self.count = count
         self.allow_from_autoenv = allow_from_autoenv
 
@@ -397,43 +513,44 @@ class ArgumentInfo(ParameterInfo):
         self,
         *,
         # ParameterInfo
-        default: Optional[Any] = None,
-        param_decls: Optional[Sequence[str]] = None,
-        callback: Optional[Callable[..., Any]] = None,
-        metavar: Optional[str] = None,
+        default: Any | None = None,
+        param_decls: Sequence[str] | None = None,
+        callback: Callable[..., Any] | None = None,
+        metavar: str | None = None,
         expose_value: bool = True,
         is_eager: bool = False,
-        envvar: Optional[Union[str, List[str]]] = None,
-        shell_complete: Optional[
-            Callable[
-                [click.Context, click.Parameter, str],
-                Union[List["click.shell_completion.CompletionItem"], List[str]],
-            ]
-        ] = None,
-        autocompletion: Optional[Callable[..., Any]] = None,
-        default_factory: Optional[Callable[[], Any]] = None,
+        envvar: str | list[str] | None = None,
+        # Note that shell_complete is not fully supported and will be removed in future versions
+        # TODO: Remove shell_complete in a future version (after 0.16.0)
+        shell_complete: Callable[
+            [click.Context, click.Parameter, str],
+            list["click.shell_completion.CompletionItem"] | list[str],
+        ]
+        | None = None,
+        autocompletion: Callable[..., Any] | None = None,
+        default_factory: Callable[[], Any] | None = None,
         # Custom type
-        parser: Optional[Callable[[str], Any]] = None,
-        click_type: Optional[click.ParamType] = None,
+        parser: Callable[[str], Any] | None = None,
+        click_type: click.ParamType | None = None,
         # TyperArgument
-        show_default: Union[bool, str] = True,
+        show_default: bool | str = True,
         show_choices: bool = True,
         show_envvar: bool = True,
-        help: Optional[str] = None,
+        help: str | None = None,
         hidden: bool = False,
         # Choice
         case_sensitive: bool = True,
         # Numbers
-        min: Optional[Union[int, float]] = None,
-        max: Optional[Union[int, float]] = None,
+        min: int | float | None = None,
+        max: int | float | None = None,
         clamp: bool = False,
         # DateTime
-        formats: Optional[List[str]] = None,
+        formats: list[str] | None = None,
         # File
-        mode: Optional[str] = None,
-        encoding: Optional[str] = None,
-        errors: Optional[str] = "strict",
-        lazy: Optional[bool] = None,
+        mode: str | None = None,
+        encoding: str | None = None,
+        errors: str | None = "strict",
+        lazy: bool | None = None,
         atomic: bool = False,
         # Path
         exists: bool = False,
@@ -443,9 +560,9 @@ class ArgumentInfo(ParameterInfo):
         readable: bool = True,
         resolve_path: bool = False,
         allow_dash: bool = False,
-        path_type: Union[None, Type[str], Type[bytes]] = None,
+        path_type: None | type[str] | type[bytes] = None,
         # Rich settings
-        rich_help_panel: Union[str, None] = None,
+        rich_help_panel: str | None = None,
     ):
         super().__init__(
             default=default,
@@ -521,3 +638,14 @@ class DeveloperExceptionConfig:
         self.pretty_exceptions_enable = pretty_exceptions_enable
         self.pretty_exceptions_show_locals = pretty_exceptions_show_locals
         self.pretty_exceptions_short = pretty_exceptions_short
+
+
+class TyperPath(click.Path):
+    # Overwrite Click's behaviour to be compatible with Typer's autocompletion system
+    def shell_complete(
+        self, ctx: click.Context, param: click.Parameter, incomplete: str
+    ) -> list[click.shell_completion.CompletionItem]:
+        """Return an empty list so that the autocompletion functionality
+        will work properly from the commandline.
+        """
+        return []
